#ifndef GROWBUF_H
#define GROWBUF_H

/*
  growbuf - Buffer that can always be appended to and iterator.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: growbuf.h,v 1.2 2004/01/08 03:47:43 martins Exp $
  
  */

#include <stddef.h>
#include <stdbool.h>

typedef struct {
	unsigned char * data;
	size_t len;
	size_t max;
	size_t chunk_size;
} growbuf;

growbuf *
growbuf_new(size_t  /* chunk size */
	);

void
growbuf_free(growbuf *   /* buf */
	);

static inline unsigned char *
growbuf_data(growbuf * buf)
{
	return buf->data;
}

static inline size_t
growbuf_length(growbuf * buf)
{
	return buf->len;
}

growbuf *
growbuf_append(growbuf *,        /* buf */
	       unsigned char *,  /* data */
	       size_t            /* data_len */
	);

typedef struct {
	growbuf * buf;
	off_t     pos;
} growbuf_iter;


/* New growbuf_iter, pos 0 */
extern growbuf_iter *
growbuf_iter_new(growbuf*);

/* Free growbuf, growbuf_iter */
extern void
growbuf_iter_free(growbuf_iter*);

/* Have at least N more */
static inline bool
growbuf_iter_have(growbuf_iter * i, off_t n)
{
	return (i->pos + n) <= growbuf_length(i->buf);
}

/* At least 1 more */
static inline bool
growbuf_iter_more(growbuf_iter * i)
{
	return growbuf_iter_have(i, 1);
}

/* These all have the precondition that enough characters are available. */

/* Current char */
static inline unsigned char *
growbuf_iter_current(growbuf_iter * i){
	return &(growbuf_data(i->buf))[i->pos];
}

/* Move N forward chars */
static inline void
growbuf_iter_move(growbuf_iter * i, off_t n)
{
	i->pos += n;
}

/* Move to next char */
static inline void
growbuf_iter_next(growbuf_iter * i)
{
	growbuf_iter_move(i, 1);
}

#endif
