# Hey Emacs, this is a -*- shell-script -*- !!!

# lsvpd-functions.bash - Shared support functions.

# This file is part of the Linux lsvpd package.

# (C) Copyright IBM Corp. 2002, 2003, 2004

# Maintained by  Martin Schwenke <martins@au.ibm.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
# $Id: lsvpd-functions.bash,v 1.22 2004/02/18 03:27:28 martins Exp $

function dec2hex ()
{
    printf "%x\n" "$1"
}

function hex2dec ()
{
    # Using implicit arithmetic conversion is faster than printf?
    case "$1" in
	0x*) echo $((  $1)) ;;
	*)   echo $((0x$1)) ;;
    esac
}

function dirname ()
{
    local yip=${1%/}
    echo "${yip%/*}"
}

function basename ()
{
    local yip=${1%/}
    echo "${yip##*/}"
}

# Very simple version of readlink that is easily tricked.  However,
# it does what we need it to, since all of our cases are simple.
function readlink ()
{
    if [ -n "$1" -a -h "$1" ] ; then
	t=$(ls -l "$1")
	echo "${t#* -> }" 
    fi
}

function unpack_device_tree_archive ()
{
    local tgz="$1"

    local dbarchivedir="/tmp/lsvpd$$"
    trap "chmod -R +w $dbarchivedir ; rm -rf $dbarchivedir" 0 1 2 3 13 15
    mkdir -p "$dbarchivedir"

    (cd "$dbarchivedir" && tar xzf -) <"$tgz" >/dev/null 2>&1
    db=$(echo "${dbarchivedir}/"*)
    # Check for system-id is for backward compatibility.
    if [ ! -d "${db}/bus" -a ! -f "${db}/system-id" ] ; then
	echo "\"$tgz\" is not a valid device-tree archive" 1>&2
	usage
    fi
}

function sanitise ()
{
    echo "$@" | sed -e 's/^[ \t]*//' -e 's/[ \t]*$//'
}

function hex2ascii ()
{
    local hex="$1"

    local i
    for i in $(echo $hex | sed -e 's/\(..\)/\1 /g') ; do
	printf "\x$i"
    done
}

function hex2substring ()
{
    local hex="$1"
    local start="$2"
    local length="$3"

    local width=2

    sanitise $(hex2ascii ${hex:$(($width * $start)):$(($width * $length))})
}

######################################################################

function multiplex ()
{
    local func="$1" ; shift
    local type="$1" ; shift

    local f="${func}_${type}"
    local t=$(type -t "$f")
    if [ "$t" = "function" ] ; then
	"$f" "$@"
    else
	f="${func}_DEFAULT"
	local t=$(type -t "$f")
	if [ "$t" = "function" ] ; then
	    "$f" "$type" "$@"
	else
	    debug "No function defined for \"${func}\" \"${type}\""
	fi
    fi
}

function make_multiplexed ()
{
    eval "function $1 () { debug \"$1 \$@\" ; multiplex $1 \"\$@\" ; }"
}
