/*
  ibm_vpd_render - Convert IBM VPD chunks to text.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: ibm_vpd_render.c,v 1.10 2004/02/16 03:39:16 martins Exp $
*/
  
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>

#include "pci_vpd_to_txt.h"
#include "growbuf.h"
#include "util.h"

static void
usage()
{
	fprintf(stderr, "usage: %s file\n", vpd_program_name);
	exit(1);
}

static growbuf *
ibm_vpd_to_txt(growbuf * s)
{
	growbuf * ret = growbuf_new(1024);

	growbuf_iter * i = growbuf_iter_new(s);

	while (growbuf_iter_have(i, 4)) {
		/* 4 byte, BIG-endian length. */
		uint32_t len = 0;
		int n;
		for (n = 0; n < 4; n++) {
			len = (len << 8) +
				(uint32_t) (* growbuf_iter_current(i));
			growbuf_iter_next(i);
		}
		
		if (growbuf_iter_have(i, len)) {
			growbuf * txt;
			/* Grab PCI VPD data */
			growbuf * pci_vpd =
				growbuf_append(growbuf_new(len),
					       growbuf_iter_current(i), len);
			growbuf_iter_move(i, len);
			
			/* Convert to text and append. */
			if (NULL != (txt = pci_vpd_to_txt(pci_vpd))) {
				char * fc = "*FC ????????\n";
				growbuf_append(ret, fc, strlen(fc));
				growbuf_append(ret,
					       growbuf_data(txt),
					       growbuf_length(txt));
				growbuf_free(txt);
			}

			growbuf_free(pci_vpd);
		} else {
			break;
		}
	}

	growbuf_iter_free(i);

	return ret;
}

int
main(int argc, char ** argv)
{
	growbuf * vpd;
	growbuf * txt;

	char * data;
	size_t len;
	int err;

	vpd_program_name = argv[0];

	if (argc != 2) {
		usage();
	}

	if (!file_read_contents(argv[1], &data, &len, &err)) {
		die2(strerror(err), argv[1]);
	}

	vpd = growbuf_append(growbuf_new(len), data, len);
	free(data);

	txt = ibm_vpd_to_txt(vpd);

	growbuf_free(vpd);

	fwrite(growbuf_data(txt), growbuf_length(txt), 1, stdout);

	growbuf_free(txt);

	return 0;
}
