/*
  scsi_vpd_std - Extract standard SCSI VPD.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: scsi_vpd_common.c,v 1.9 2004/01/08 04:00:08 martins Exp $
*/
  
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "scsi_vpd_common.h"
#include "util.h"

static char *
hexify(const char * s, size_t len)
{
	int i;
	char * ret = malloc(len*2+1);
	for (i = 0; i < len ; i++) {
		snprintf(&ret[i*2], 3, "%.2X", (unsigned char) s[i]);
	}

	return ret;
}

static void
strstrip(char * s)
{
	char *b, *t, *e;

	b = s;
	while (isspace(*b)) {
		b++;
	}

	e = t = b;
	while (isprint(*t)) {
		if (!isspace(*t)) {
			e = t;
		}
		t++;
	}
	e++;

	memmove(s, b, e - b);
	s[e - b] = '\0';
}

void
dump_field(const char * dir,
	   const char * key,
	   const char * data,
	   size_t       len,
	   const char * dtype)
{
	/*
	 * Many fields will be whitespace-stripped below, so we'll
	 * need to make sure the string is NUL-terminated.  This means
	 * we need to take a copy of the string.
	 */
	char * ldata;

	/* Certain fields will require some munging. */
	/* Convert RL, Z0, Z7 to hex. */
	if (((strcmp(key, "RL") == 0) && (strcmp(dtype, "disk") == 0)) ||
	    (strcmp(key, "Z0") == 0) ||
	    (strcmp(key, "Z7") == 0)) {
		/* hexify returns a new NUL-terminated string. */
		ldata = hexify(data, len);
	} else {
		/* Copy data and NUL-terminate if necessary. */
		ldata = malloc(len+1);
		memmove(ldata, data, len);
		ldata[len] = '\0';
	}

	/* This also terminates at the first NUL. */
	strstrip(ldata);

	/* Write data to file. */
	file_relative_write_contents(dir, key, ldata, strlen(ldata));

	free(ldata);
}

